/*
	cnspSalesAccountRename:    Corrin Sales Account Rename
	@OldAccount:               Old Account Code
	@NewAccount:               New Account Code
	@UserId:                   User ID
	@Merge:                    Merge Accounts
	@SPM:                      Update SPM
	@DDBacs:                   Update Direct Debits (Bacs)
	@CorrinInternal:           Update Corrin Internal
	Returns:                   0 for success
	Copyright:                 (c) 2024 Corrin Software Products Ltd
*/

IF EXISTS (SELECT * from dbo.sysobjects WHERE id = object_id(N'[dbo].[cnspSalesAccountRename]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[cnspSalesAccountRename]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[cnspSalesAccountRename]
	@OldAccount     VARCHAR(8),
	@NewAccount     VARCHAR(8),
	@UserId         VARCHAR(10),
	@Merge          BIT,
	@SPM            BIT,
	@DDBacs         BIT,
	@CorrinInternal BIT
AS
	BEGIN
		DECLARE @ErrorSave INT = 0,
			@DateTime DATETIME = GetDate(),
			@Result INT,
			@NextId INT;
		BEGIN TRANSACTION;
		UPDATE arline SET at_account = @NewAccount 
			FROM arline 
			JOIN arhead ON arline.at_entry = arhead.ae_entry
			WHERE arline.at_account = @OldAccount AND arhead.ae_type IN (3, 4);
		UPDATE atran SET at_account = @NewAccount WHERE at_account = @OldAccount AND at_type IN (3, 4);
		UPDATE cdisc SET sd_code = @NewAccount+RIGHT(sd_code, 4) WHERE LEFT(sd_code, 8) = @OldAccount;	
		UPDATE chead SET cx_saleacc = @NewAccount WHERE cx_saleacc = @OldAccount;
		UPDATE ctrack SET ck_account = @NewAccount WHERE ck_account = @OldAccount AND (ck_delivry <> '' OR ck_invoice <> '');
		UPDATE ctran SET ct_account = @NewAccount WHERE ct_account = @OldAccount AND ct_type IN ('A', 'I' , 'T', 'W', 'R', 'N');
		UPDATE doline SET do_socust = @NewAccount WHERE do_socust = @OldAccount;
		UPDATE ftrex SET fr_dspacc = @NewAccount WHERE fr_dspacc = @OldAccount;
		UPDATE hsanal SET sa_account = @NewAccount WHERE sa_account = @OldAccount;
		UPDATE hsanal SET sa_daccnt = @NewAccount WHERE sa_daccnt = @OldAccount;
		UPDATE ihead SET ih_account = @NewAccount WHERE ih_account = @OldAccount;
		UPDATE ihead SET ih_postacc = @NewAccount WHERE ih_postacc = @OldAccount;
		UPDATE ialloc SET it_account = @NewAccount WHERE it_account = @OldAccount;
		UPDATE jchead SET jh_account = @NewAccount WHERE jh_account = @OldAccount;
		UPDATE jcntr SET jc_account = @NewAccount WHERE jc_account = @OldAccount;
		UPDATE jehead SET jh_account = @NewAccount WHERE jh_account = @OldAccount;
		UPDATE lcprod SET lc_snacct = @NewAccount WHERE lc_snacct = @OldAccount;
		UPDATE ldefa SET lf_account = @NewAccount WHERE lf_account = @OldAccount;
		UPDATE ldocs SET ld_account = @NewAccount WHERE ld_account = @OldAccount;
		UPDATE lihead SET lh_account = @NewAccount WHERE lh_account = @OldAccount;
		UPDATE lihead SET lh_postacc = @NewAccount WHERE lh_postacc = @OldAccount;
		UPDATE iwxhead SET ii_account = @NewAccount WHERE ii_account = @OldAccount;
		UPDATE salloc SET al_account = @NewAccount WHERE al_account = @OldAccount;
		UPDATE sanal SET sa_account = @NewAccount WHERE sa_account = @OldAccount;
		UPDATE sanal SET sa_daccnt = @NewAccount WHERE sa_daccnt = @OldAccount;
		UPDATE sdealloch SET sf_account = @NewAccount WHERE sf_account = @OldAccount;
		UPDATE sladlett SET sb_account = @NewAccount WHERE sb_account = @OldAccount;
		UPDATE slcmacts SET sm_customr = @NewAccount WHERE sm_customr = @OldAccount;
		UPDATE stran SET st_account = @NewAccount WHERE st_account = @OldAccount;
		UPDATE stran SET st_delacc = @NewAccount WHERE st_delacc = @OldAccount;
		UPDATE zcontacts SET zc_account = @NewAccount WHERE zc_account = @OldAccount AND zc_module = 'S';
		UPDATE znotes SET zn_account = @NewAccount WHERE zn_account = @OldAccount AND zn_module = 'S';
		UPDATE zvtran SET va_account = @NewAccount WHERE va_account = @OldAccount AND va_source = 'S';
		UPDATE zvtran SET va_laccnt = @NewAccount WHERE va_laccnt = @OldAccount AND va_source = 'S';
		UPDATE zwximsdh SET zsh_account = @NewAccount WHERE zsh_account = @OldAccount;
		IF @Merge = 1
		BEGIN
			DECLARE @Trnover DECIMAL(15,2), @LastInv DATETIME, @LastRec DATETIME,
				@CurrBal DECIMAL(15,2), @OrdrBal DECIMAL(15,2), @NextPay DECIMAL(3,0);
			SELECT @Trnover = sn_trnover, @LastInv = sn_lastinv, @LastRec = sn_lastrec,
				@CurrBal = sn_currbal, @OrdrBal = sn_ordrbal, @NextPay = sn_nextpay
				FROM sname 
				WHERE sn_account = @OldAccount;
			UPDATE sname
				SET sn_trnover = 0,
				sn_currbal = 0,
				sn_ordrbal = 0,
				sn_dormant = 1,
				[state] = 2
				WHERE sn_account = @OldAccount;
			UPDATE sprfls
				SET [state] = 2
				WHERE sc_account = @OldAccount;
			UPDATE sterms
				SET [state] = 2
				WHERE sr_account = @OldAccount;
			EXECUTE dbo.pgspTableIDGet @Result OUTPUT, N'znotes', 1, @NextId OUTPUT; 
			IF @Result = 0
			BEGIN
				INSERT INTO znotes (id, zn_module, zn_account, sq_user, sq_date, sq_time, zn_note, zn_actcomp, 
					zn_compdte, zn_priority, zn_subject)
					VALUES (@NextId, 'S', @OldAccount, @UserId, Cast(@DateTime AS DATE), Cast(@DateTime AS TIME), 
					'Account transferred to '+Trim(@NewAccount), 1, Cast(@DateTime AS DATE), 1, 'Account Rename');
			END;
			UPDATE sname
				SET sn_trnover = sn_trnover + @Trnover,
				sn_lastinv = GREATEST(sn_lastinv, @LastInv),
				sn_lastrec = GREATEST(sn_lastrec, @LastRec),
				sn_currbal = sn_currbal + @CurrBal,
				sn_ordrbal = sn_ordrbal + @OrdrBal,
				sn_nextpay = GREATEST(sn_nextpay, @NextPay),
				sn_lupdate = GETDATE()
				WHERE sn_account = @NewAccount;
			EXECUTE dbo.pgspTableIDGet @Result OUTPUT, N'znotes', 1, @NextId OUTPUT; 
			IF @Result = 0
			BEGIN
				INSERT INTO znotes (id, zn_module, zn_account, sq_user, sq_date, sq_time, zn_note, zn_actcomp, 
					zn_compdte, zn_priority, zn_subject)
					VALUES (@NextId, 'S', @NewAccount, @UserId, Cast(@DateTime AS DATE), Cast(@DateTime AS TIME), 
					'Account details added from '+Trim(@OldAccount), 1, Cast(@DateTime AS DATE), 1, 'Account Rename');
			END;
			SELECT * 
				INTO #TempSHist
				FROM shist
				WHERE si_account = @OldAccount AND [state] = 1;
			UPDATE #TempSHist
				SET si_account = @NewAccount;
			CREATE TABLE #UpdatedSHist (id INT);
			MERGE shist
				USING #TempSHist AS Source
				ON shist.si_account = Source.si_account AND shist.si_age = Source.si_age
				WHEN MATCHED THEN
					UPDATE SET
					shist.si_turn = shist.si_turn + Source.si_turn,
					shist.si_vat = shist.si_vat + Source.si_vat,
					shist.si_cash = shist.si_cash + Source.si_cash,
					shist.si_adjamt = shist.si_adjamt + Source.si_adjamt,
					shist.si_outbal = shist.si_outbal + Source.si_outbal,
					shist.si_avgdays = ROUND((shist.si_avgdays + Source.si_avgdays) / 2, 1),
					shist.si_current = shist.si_current + Source.si_current,
					shist.si_period1 = shist.si_period1 + Source.si_period1,
					shist.si_period2 = shist.si_period2 + Source.si_period2,
					shist.si_period3 = shist.si_period3 + Source.si_period3,
					shist.si_period4 = shist.si_period4 + Source.si_period4,
					shist.si_period5 = shist.si_period5 + Source.si_period5
				OUTPUT INSERTED.id INTO #UpdatedSHist;
			UPDATE shist
				SET [state] = 2
				WHERE id IN (SELECT id FROM #UpdatedSHist);
			UPDATE shist
				SET si_account = @NewAccount
				WHERE si_account = @OldAccount AND [state] = 1;
			IF @SPM = 1
			BEGIN
				DECLARE @OldCompId INT, @NewCompId INT, @CompNo CHAR(8);
				SELECT @OldCompId = compid
					FROM dmcomp
					WHERE customer = @OldAccount AND [state] = 1;
				IF @OldCompId > 0
				BEGIN
					SELECT @NewCompId = compid, @CompNo = comp_no
						FROM dmcomp
						WHERE customer = @NewAccount AND [state] = 1;
					IF @NewCompId > 0
					BEGIN
						UPDATE dmcomp SET
							comptype = 'Prospect',
							customer = '',
							[state] = 2
							WHERE customer = @OldAccount;
						UPDATE dmcont SET
							compid = @NewCompId,
							comp_no = @CompNo,
							customer = @NewAccount
							WHERE compid = @OldCompId;
						UPDATE dmaddr SET
							compid = @NewCompId,
							comp_no = @CompNo
							WHERE compid = @OldCompId;
					END;
					ELSE
					BEGIN
						UPDATE dmcomp SET
							customer = @NewAccount
							WHERE customer = @OldAccount;
						UPDATE dmcont SET
							customer = @NewAccount
							WHERE customer = @OldAccount;
					END;
				END;
			END;
		END;
		ELSE
		BEGIN
			UPDATE shist SET si_account = @NewAccount WHERE si_account = @OldAccount;
			UPDATE sname SET sn_account = @NewAccount WHERE sn_account = @OldAccount;
			UPDATE sname SET sn_stmntac = @NewAccount WHERE sn_stmntac = @OldAccount;
			UPDATE sname SET sn_invceac = @NewAccount WHERE sn_invceac = @OldAccount;
			UPDATE sprfls SET sc_account = @NewAccount WHERE sc_account = @OldAccount;
			UPDATE sterms SET sr_account = @NewAccount WHERE sr_account = @OldAccount;			
			IF @SPM = 1
			BEGIN
				UPDATE dmcomp SET customer = @NewAccount WHERE customer = @OldAccount;
				UPDATE dmcont SET customer = @NewAccount WHERE customer = @OldAccount;
			END;
			EXECUTE dbo.pgspTableIDGet @Result OUTPUT, N'znotes', 1, @NextId OUTPUT; 
			IF @Result = 0
			BEGIN
				INSERT INTO znotes (id, zn_module, zn_account, sq_user, sq_date, sq_time, zn_note, zn_actcomp, zn_compdte, zn_priority, zn_subject)
					VALUES (@NextId, 'S', @NewAccount, @UserId, Cast(@DateTime AS DATE), Cast(@DateTime AS TIME), 
					'Account renamed from '+Trim(@OldAccount)+' to '+Trim(@NewAccount),1, Cast(@DateTime AS DATE), 1, 'Account Rename');
			END;
		END;
		IF @DDBacs = 1
		BEGIN
			UPDATE pbaudt SET pu_account = @NewAccount WHERE pu_account = @OldAccount;
		END;
		IF @CorrinInternal = 1
		BEGIN
			UPDATE platcust SET pc_accode = @NewAccount WHERE pc_accode = @OldAccount;
			UPDATE cspprod SET prod_accod = @NewAccount WHERE prod_accod = @OldAccount;
		END;
		COMMIT TRANSACTION;
		RETURN @ErrorSave;
	END
GO